(function (exports) {
	'use strict';

	let foo = 'foo';

	exports.foo = foo;

	Object.defineProperty(exports, '__esModule', { value: true });

}(this['@scoped/npm-package'] = this['@scoped/npm-package'] || {}));
