System.register(['./hsl2hsv.js', './generated-index.js'], function (exports) {
	'use strict';
	return {
		setters: [function (module) {
			exports('hsl2hsv', module.default);
		}, function () {}],
		execute: function () {



		}
	};
});
