(function (exports) {
	'use strict';

	let foo = 'foo';

	exports.foo = foo;

	Object.defineProperty(exports, '__esModule', { value: true });

}(this['module-name-with-dashes'] = this['module-name-with-dashes'] || {}));
