'use strict';

function _interopNamespaceDefaultOnly(e) {
	return Object.freeze({__proto__: null, 'default': e});
}

Promise.resolve().then(function () { return /*#__PURE__*/_interopNamespaceDefaultOnly(require('./lib.js')); }).then(console.log);
