/* this is a banner */
/* first banner */
/* second banner */
/* 3rd banner */
(function (factory) {
	typeof define === 'function' && define.amd ? define(factory) :
	factory();
}((function () { 'use strict';

	console.log( 'hello world' );

})));
/* this is a footer */
/* first footer */
/* second footer */
/* 3rd footer */
