/* this is a banner */
/* first banner */
/* second banner */
/* 3rd banner */
System.register([], function () {
	'use strict';
	return {
		execute: function () {

			console.log( 'hello world' );

		}
	};
});
/* this is a footer */
/* first footer */
/* second footer */
/* 3rd footer */
