define(['a', 'b'], function (a, b) { 'use strict';

	function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

	var a__default = /*#__PURE__*/_interopDefaultLegacy(a);
	var b__default = /*#__PURE__*/_interopDefaultLegacy(b);

	console.log(a__default['default'], b__default['default']);

});
