System.register([], function (exports) {
	'use strict';
	return {
		execute: function () {

			const exists = exports('e', 'exists');
			const __synthetic = exports('_', { synthetic: 'synthetic' });

		}
	};
});
