'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var external = require('external');



Object.defineProperty(exports, 's', {
	enumerable: true,
	get: function () {
		return external.p;
	}
});
