'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var d = require('d');



Object.defineProperty(exports, 'b', {
	enumerable: true,
	get: function () {
		return d.d;
	}
});
