define(['exports'], function (exports) { 'use strict';

	const shared1 = 'shared1';
	const foo = 'foo1';

	var shared2 = 'shared2';
	const foo$1 = 'foo2';

	exports.foo = foo;
	exports.foo$1 = foo$1;
	exports.shared1 = shared1;
	exports.shared2 = shared2;

});
