define(['quoted\'\
\
\ \ external1', 'quoted\'\
\
\ \ external2', 'C:\\File\\Path.js'], function (quoted_____external1, quoted_____external2, Path_js) { 'use strict';

	console.log(quoted_____external1.foo, quoted_____external2.bar, Path_js.baz);

});
