import * as dep from 'dep';

console.log(dep.await);
console.log(dep.break);
console.log(dep.case);
console.log(dep.catch);
console.log(dep.class);
console.log(dep.const);
console.log(dep.continue);
console.log(dep.debugger);
console.log(dep.default);
console.log(dep.delete);
console.log(dep.do);
console.log(dep.else);
console.log(dep.enum);
console.log(dep.eval);
console.log(dep.export);
console.log(dep.extends);
console.log(dep.finally);
console.log(dep.for);
console.log(dep.function);
console.log(dep.if);
console.log(dep.implements);
console.log(dep.import);
console.log(dep.in);
console.log(dep.instanceof);
console.log(dep.interface);
console.log(dep.let);
console.log(dep.new);
console.log(dep.null);
console.log(dep.package);
console.log(dep.private);
console.log(dep.protected);
console.log(dep.public);
console.log(dep.return);
console.log(dep.static);
console.log(dep.super);
console.log(dep.switch);
console.log(dep.throw);
console.log(dep.try);
console.log(dep.typeof);
console.log(dep.undefined);
console.log(dep.var);
console.log(dep.void);
console.log(dep.while);
console.log(dep.with);
console.log(dep.yield);
