System.register('myBundle', ['babel-polyfill'], function (exports) {
	'use strict';
	return {
		setters: [function () {}],
		execute: function () {

			var main = exports('default', new WeakMap());

		}
	};
});
