var hello = 'world';

var hello_1 = hello;

var myBasePkg = /*#__PURE__*/Object.defineProperty({
	hello: hello_1
}, '__esModule', {value: true});

export default myBasePkg;
export { hello_1 as hello };
