System.register(['./generated-name.js', './generated-secondName.js', './generated-name2.js', './mainChunk.js'], function () {
	'use strict';
	return {
		setters: [function () {}, function () {}, function () {}, function () {}],
		execute: function () {



		}
	};
});
