define(['./generated-geometry'], function (geometry) { 'use strict';

	var mod = /*#__PURE__*/Object.freeze({
		__proto__: null,
		volume: geometry.volume,
		geometry: geometry.geometry
	});

	console.log(mod);

});
