System.register(['./generated-dep.js', 'external'], function (exports) {
	'use strict';
	return {
		setters: [function () {}, function (module) {
			exports('dep', module.asdf);
		}],
		execute: function () {



		}
	};
});
