System.register(['./generated-main2.js'], function (exports) {
	'use strict';
	return {
		setters: [function (module) {
			var _setter = {};
			_setter.a = module.a;
			_setter.b = module.b;
			exports(_setter);
		}],
		execute: function () {



		}
	};
});
