System.register(['./generated-manual-outer.js', './generated-manual-middle.js', './generated-manual-inner.js'], function () {
	'use strict';
	return {
		setters: [function () {}, function () {}, function () {}],
		execute: function () {

			console.log('main');

		}
	};
});
