const x = 'code';
console.log('should remain impure');

console.log('should remain impure');

console.log('code');
console.log('should remain impure');

console.log('code');
console.log('should remain impure');
console.log('should remain impure');

console.log('code');
console.log('should remain impure');

console.log('code'),
console.log('should remain impure');

console.log('code'),
console.log('should remain impure');

console.log('should remain impure');

console.log('code');
console.log('should remain impure');

 console.log('should remain impure');

 console.log('should remain impure') ;

console.log('code');
console.log('should remain impure', x);


{
	console.log('should remain impure');
}
