const object = {};
object.propertyIsEnumerable( 'toString' );
{}.propertyIsEnumerable( 'toString' );
{}.propertyIsEnumerable( 'toString' ).valueOf();

{}.hasOwnProperty( 'toString' ).valueOf();
{}.isPrototypeOf( {} ).valueOf();
{}.propertyIsEnumerable( 'toString' ).valueOf();
{}.toLocaleString().trim();
{}.toString().trim();
{}.valueOf();
