import { lib } from './main.js';

var component = { lib, someExport: lib.someExport };

var component$1 = /*#__PURE__*/Object.freeze(/*#__PURE__*/Object.assign(/*#__PURE__*/Object.create(null), component, {
	'default': component
}));

export { component$1 as c };
