module.exports = {
	description: 'throws when setting preserveEntrySignatures to false',
	options: {
		strictDeprecations: false,
		input: ['main.js'],
		preserveModules: true,
		preserveEntrySignatures: false
	},
	generateError: {
		code: 'INVALID_OPTION',
		message:
			'Setting "preserveEntrySignatures" to "false" is not supported for "output.preserveModules".'
	}
};
