import { l as lib } from './generated-main.js';

var component = { lib, lib2: lib.named, lib3: lib.named.named };

var component$1 = /*#__PURE__*/Object.freeze(/*#__PURE__*/Object.assign(/*#__PURE__*/Object.create(null), component, {
	'default': component
}));

export { component$1 as c };
